@@ -18,9 +18,6 @@ use alloc::string::String;
 #[cfg(feature = "serde_support")]
 use serde::{de::Deserializer, ser::Serializer, Deserialize, Serialize};
 
-// The instances of `#[rustfmt::skip]` specifically above the `const` impls below are currently
-// necessary, as otherwise rustfmt just silently deletes the `const` keyword.
-
 impl<const N: usize> Add<&str> for StaticString<N> {
   type Output = Self;
 
@@ -38,7 +35,6 @@ impl<const N: usize> AddAssign<&str> for StaticString<N> {
   }
 }
 
-#[rustfmt::skip]
 impl<const N: usize> const AsMut<str> for StaticString<N> {
   #[inline(always)]
   fn as_mut(&mut self) -> &mut str {
@@ -46,7 +42,6 @@ impl<const N: usize> const AsMut<str> for StaticString<N> {
   }
 }
 
-#[rustfmt::skip]
 impl<const N: usize> const AsRef<str> for StaticString<N> {
   #[inline(always)]
   fn as_ref(&self) -> &str {
@@ -54,7 +49,6 @@ impl<const N: usize> const AsRef<str> for StaticString<N> {
   }
 }
 
-#[rustfmt::skip]
 impl<const N: usize> const AsRef<[u8]> for StaticString<N> {
   #[inline(always)]
   fn as_ref(&self) -> &[u8] {
@@ -62,7 +56,6 @@ impl<const N: usize> const AsRef<[u8]> for StaticString<N> {
   }
 }
 
-#[rustfmt::skip]
 impl<const N: usize> const Borrow<str> for StaticString<N> {
   #[inline(always)]
   fn borrow(&self) -> &str {
@@ -70,7 +63,6 @@ impl<const N: usize> const Borrow<str> for StaticString<N> {
   }
 }
 
-#[rustfmt::skip]
 impl<const N: usize> const BorrowMut<str> for StaticString<N> {
   #[inline(always)]
   fn borrow_mut(&mut self) -> &mut str {
@@ -78,6 +70,15 @@ impl<const N: usize> const BorrowMut<str> for StaticString<N> {
   }
 }
 
+impl<const N: usize> Clone for StaticString<N> {
+  #[inline(always)]
+  fn clone(&self) -> Self {
+    Self {
+      vec: self.vec.clone(),
+    }
+  }
+}
+
 impl<const N: usize> Debug for StaticString<N> {
   #[inline(always)]
   fn fmt(&self, f: &mut Formatter) -> fmt::Result {
@@ -88,7 +89,6 @@ impl<const N: usize> Debug for StaticString<N> {
   }
 }
 
-#[rustfmt::skip]
 impl<const N: usize> const Default for StaticString<N> {
   #[inline(always)]
   fn default() -> Self {
@@ -96,7 +96,6 @@ impl<const N: usize> const Default for StaticString<N> {
   }
 }
 
-#[rustfmt::skip]
 impl<const N: usize> const Deref for StaticString<N> {
   type Target = str;
 
@@ -106,7 +105,6 @@ impl<const N: usize> const Deref for StaticString<N> {
   }
 }
 
-#[rustfmt::skip]
 impl<const N: usize> const DerefMut for StaticString<N> {
   #[inline(always)]
   fn deref_mut(&mut self) -> &mut Self::Target {
@@ -247,7 +245,6 @@ impl<const N: usize> IndexMut<RangeFrom<usize>> for StaticString<N> {
   }
 }
 
-#[rustfmt::skip]
 impl<const N: usize> const Index<RangeFull> for StaticString<N> {
   type Output = str;
 
@@ -257,7 +254,6 @@ impl<const N: usize> const Index<RangeFull> for StaticString<N> {
   }
 }
 
-#[rustfmt::skip]
 impl<const N: usize> const IndexMut<RangeFull> for StaticString<N> {
   #[inline(always)]
   fn index_mut(&mut self, _index: RangeFull) -> &mut str {
