@@ -1,15 +1,11 @@
 use super::{StaticString, StringError};
 
-pub(crate) trait IntoLossy<T>: Sized {
-  fn into_lossy(self) -> T;
-}
-
 /// Unsafely marks a branch as unreachable.
 #[inline(always)]
 #[allow(unused_variables)]
 pub(crate) unsafe fn never(s: &str) -> ! {
   #[cfg(debug_assertions)]
-  panic!("{}", s);
+  core::panic!("{}", s);
   #[cfg(not(debug_assertions))]
   core::hint::unreachable_unchecked()
 }
@@ -25,7 +21,7 @@ const MAX_THREE_B: u32 = 0x10000;
 
 /// Encodes `character` into `string` at the specified position.
 #[inline(always)]
-pub(crate) unsafe fn encode_char_utf8_unchecked<const N: usize>(
+pub(crate) const unsafe fn encode_char_utf8_unchecked<const N: usize>(
   string: &mut StaticString<N>,
   character: char,
   index: usize,
@@ -85,7 +81,7 @@ pub(crate) unsafe fn shift_left_unchecked<const N: usize>(
 
 /// Returns an error if `size` is greater than `limit`.
 #[inline(always)]
-pub fn is_inside_boundary(size: usize, limit: usize) -> Result<(), StringError> {
+pub const fn is_inside_boundary(size: usize, limit: usize) -> Result<(), StringError> {
   match size <= limit {
     false => Err(StringError::OutOfBounds),
     true => Ok(()),
@@ -121,22 +117,6 @@ pub(crate) fn truncate_str(slice: &str, size: usize) -> &str {
   }
 }
 
-impl IntoLossy<u8> for usize {
-  #[allow(clippy::cast_possible_truncation)]
-  #[inline]
-  fn into_lossy(self) -> u8 {
-    self as u8
-  }
-}
-
-impl IntoLossy<u8> for u32 {
-  #[allow(clippy::cast_possible_truncation)]
-  #[inline]
-  fn into_lossy(self) -> u8 {
-    self as u8
-  }
-}
-
 #[cfg(test)]
 mod tests {
   use super::*;
