@@ -317,6 +317,18 @@ impl<'a, T: 'a + Copy, const N: usize> ExtendEx<&'a T, core::slice::Iter<'a, T>>
   }
 }
 
+impl<T: Copy, const N: usize> ExtendEx<T, core::array::IntoIter<T, N>> for StaticVec<T, N> {
+  #[inline(always)]
+  fn extend_ex(&mut self, iter: core::array::IntoIter<T, N>) {
+    self.extend_from_slice(iter.as_slice());
+  }
+
+  #[inline(always)]
+  fn from_iter_ex(iter: core::array::IntoIter<T, N>) -> Self {
+    Self::new_from_slice(iter.as_slice())
+  }
+}
+
 impl<'a, T: 'a + Copy, const N: usize> const ExtendEx<&'a T, &'a [T]> for StaticVec<T, N> {
   #[inline(always)]
   fn extend_ex(&mut self, iter: &'a [T]) {
@@ -602,31 +614,32 @@ impl<T, const N: usize> const IndexMut<RangeFull> for StaticVec<T, N> {
   }
 }
 
-impl<T, const N: usize> Index<RangeInclusive<usize>> for StaticVec<T, N> {
+impl<T, const N: usize> const Index<RangeInclusive<usize>> for StaticVec<T, N> {
   type Output = [T];
   /// Asserts that the lower bound of `index` is less than or equal to its upper bound,
   /// and that its upper bound is less than the current length of the StaticVec,
   /// and if so returns a constant reference to a slice of elements `index.start()..=index.end()`.
   #[inline(always)]
   fn index(&self, index: RangeInclusive<usize>) -> &Self::Output {
-    assert!(index.start() <= index.end() && *index.end() < self.length);
-    slice_from_raw_parts(
-      unsafe { self.ptr_at_unchecked(*index.start()) },
-      (index.end() + 1) - index.start(),
-    )
+    let start = *index.start();
+    let end = *index.end();
+    assert!(start <= end && end < self.length);
+    slice_from_raw_parts(unsafe { self.ptr_at_unchecked(start) }, (end + 1) - start)
   }
 }
 
-impl<T, const N: usize> IndexMut<RangeInclusive<usize>> for StaticVec<T, N> {
+impl<T, const N: usize> const IndexMut<RangeInclusive<usize>> for StaticVec<T, N> {
   /// Asserts that the lower bound of `index` is less than or equal to its upper bound,
   /// and that its upper bound is less than the current length of the StaticVec,
   /// and if so returns a mutable reference to a slice of elements `index.start()..=index.end()`.
   #[inline(always)]
   fn index_mut(&mut self, index: RangeInclusive<usize>) -> &mut Self::Output {
-    assert!(index.start() <= index.end() && *index.end() < self.length);
+    let start = *index.start();
+    let end = *index.end();
+    assert!(start <= end && end < self.length);
     slice_from_raw_parts_mut(
-      unsafe { self.mut_ptr_at_unchecked(*index.start()) },
-      (index.end() + 1) - index.start(),
+      unsafe { self.mut_ptr_at_unchecked(start) },
+      (end + 1) - start,
     )
   }
 }
