@@ -83,6 +83,16 @@ where T: Copy {
   }
 }
 
+/// An internal convenience function for incrementing mutable ZST pointers by usize offsets.
+pub(crate) const fn zst_ptr_add_mut<T>(ptr: *mut T, offset: usize) -> *mut T {
+  unsafe { (ptr as usize + offset) as *mut T }
+}
+
+/// An internal convenience function for incrementing immutable ZST pointers by usize offsets.
+pub(crate) const fn zst_ptr_add<T>(ptr: *const T, offset: usize) -> *const T {
+  unsafe { (ptr as usize + offset) as *const T }
+}
+
 /// A version of the default `partial_cmp` implementation with a more flexible function signature.
 #[inline]
 pub(crate) fn partial_compare<T1, T2: PartialOrd<T1>>(
@@ -177,7 +187,7 @@ pub(crate) const fn slice_from_raw_parts<'a, T>(data: *const T, length: usize) -
     is_aligned_and_not_null(data),
     "Attempted to create an unaligned or null slice!"
     */
-    // See comment starting at line 154 for more info about what's going on here. Note that the
+    // See comment starting at line 165 for more info about what's going on here. Note that the
     // alignment check is not actually a concern for our use case anyways, since we only call this
     // function with known-valid pointers to initialized elements of a StaticVec's internal array.
     !data.is_null(),
@@ -198,7 +208,7 @@ pub(crate) const fn slice_from_raw_parts_mut<'a, T>(data: *mut T, length: usize)
     is_aligned_and_not_null(data),
     "Attempted to create an unaligned or null slice!"
     */
-    // See comment starting at line 154 for more info about what's going on here. Note that the
+    // See comment starting at line 165 for more info about what's going on here. Note that the
     // alignment check is not actually a concern for our use case anyways, since we only call this
     // function with known-valid pointers to initialized elements of a StaticVec's internal array.
     !data.is_null(),
